"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResolveMiddleware = createResolveMiddleware;
exports.default = void 0;

var _assign = _interopRequireDefault(require("@babel/runtime/core-js/object/assign"));

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _action_types = require("./action_types");

var _utils = require("./utils");

var _actions = _interopRequireDefault(require("./actions"));

var _subscribe_adapter = _interopRequireDefault(require("./subscribe_adapter"));

var _send_command = _interopRequireDefault(require("./send_command"));

var _create_actions = _interopRequireDefault(require("./create_actions"));

var _mock_subscribe_adapter = _interopRequireDefault(require("./mock_subscribe_adapter"));

var _subscribe_view_model = _interopRequireDefault(require("./subscribe_view_model"));

var _unsubscribe_view_model = _interopRequireDefault(require("./unsubscribe_view_model"));

var _subscribe_read_model = _interopRequireDefault(require("./subscribe_read_model"));

var _unsubscribe_read_model = _interopRequireDefault(require("./unsubscribe_read_model"));

var REFRESH_TIMEOUT = 1000;
var isClient = typeof window !== 'undefined';

function createResolveMiddleware(_ref) {
  var _ref$viewModels = _ref.viewModels,
      viewModels = _ref$viewModels === void 0 ? [] : _ref$viewModels,
      _ref$readModels = _ref.readModels,
      readModels = _ref$readModels === void 0 ? [] : _ref$readModels,
      _ref$aggregates = _ref.aggregates,
      aggregates = _ref$aggregates === void 0 ? [] : _ref$aggregates,
      _ref$subscribeAdapter = _ref.subscribeAdapter,
      subscribeAdapter = _ref$subscribeAdapter === void 0 ? _subscribe_adapter.default : _ref$subscribeAdapter,
      origin = _ref.origin,
      rootPath = _ref.rootPath;
  var subscribers = {
    viewModels: {},
    aggregateIds: {}
  };
  var requests = {};
  var loading = {};
  var readModelSubscriptions = {};
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator2.default)(viewModels), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref3 = _step.value;
      var name = _ref3.name;
      loading[name] = {};
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var aggregateActions = {};
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = (0, _getIterator2.default)(aggregates), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var aggregate = _step2.value;
      (0, _assign.default)(aggregateActions, (0, _create_actions.default)(aggregate));
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var orderedFetch = (0, _utils.createOrderedFetch)();
  return function (store) {
    var adapter = (isClient ? subscribeAdapter : _mock_subscribe_adapter.default)({
      origin: origin,
      rootPath: rootPath
    });
    adapter.onEvent(function (event) {
      return store.dispatch(event);
    });
    adapter.onDisconnect(function (error) {
      store.dispatch(_actions.default.disconnect(error));
      (0, _keys.default)(readModelSubscriptions).forEach(function (subscriptionKey) {
        readModelSubscriptions[subscriptionKey].refresh();
      });
    });
    var context = {
      origin: origin,
      rootPath: rootPath,
      adapter: adapter,
      viewModels: viewModels,
      readModels: readModels,
      aggregates: aggregates,
      aggregateActions: aggregateActions,
      store: store,
      subscribers: subscribers,
      requests: requests,
      readModelSubscriptions: readModelSubscriptions,
      orderedFetch: orderedFetch
    };
    Object.defineProperty(store.getState, 'isLoadingViewModel', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: function value(viewModelName, aggregateId) {
        return !!loading[viewModelName][aggregateId];
      }
    });
    Object.defineProperty(store.getState, 'aggregateActions', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: aggregateActions
    });
    store.dispatch(_actions.default.provideViewModels(viewModels));

    if (!isClient) {
      return function (next) {
        return function (action) {
          return next(action);
        };
      };
    }

    var hotModuleReplacementId;
    return function (next) {
      return function (action) {
        switch (action.type) {
          case _action_types.HOT_MODULE_REPLACEMENT:
            {
              if (!hotModuleReplacementId) {
                hotModuleReplacementId = action.hotModuleReplacementId;
              }

              if (hotModuleReplacementId !== action.hotModuleReplacementId) {
                window.location.reload();
              }

              break;
            }

          case _action_types.SUBSCRIBE_VIEWMODEL:
            {
              var viewModelName = action.viewModelName,
                  aggregateId = action.aggregateId;
              loading[viewModelName][aggregateId] = true;
              (0, _subscribe_view_model.default)(context, action).catch(function (error) {
                return setTimeout(function () {
                  // eslint-disable-next-line no-console
                  console.error(error);
                  store.dispatch(action);
                }, REFRESH_TIMEOUT);
              });
              break;
            }

          case _action_types.UNSUBSCRIBE_VIEWMODEL:
            {
              var _viewModelName = action.viewModelName,
                  _aggregateId = action.aggregateId;
              delete loading[_viewModelName][_aggregateId];
              (0, _unsubscribe_view_model.default)(context, action);
              break;
            }

          case _action_types.SUBSCRIBE_READMODEL:
            {
              (0, _subscribe_read_model.default)(context, action);
              break;
            }

          case _action_types.UNSUBSCRIBE_READMODEL:
            {
              (0, _unsubscribe_read_model.default)(context, action);
              break;
            }

          case _action_types.SEND_COMMAND:
            {
              (0, _send_command.default)(context, action);
              break;
            }

          default:
        }

        return next(action);
      };
    };
  };
}

var _default = createResolveMiddleware;
exports.default = _default;