"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createOrderedFetch = exports.getAggregateIds = exports.getEventTypes = exports.makeLateResolvingPromise = exports.delay = exports.getKey = exports.getRootBasedUrl = exports.checkRequiredFields = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _promise = _interopRequireDefault(require("@babel/runtime/core-js/promise"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var checkRequiredFields = function checkRequiredFields(obj, beforeWarnings, afterWarnings) {
  var warningMessages = (0, _keys.default)(obj).map(function (fieldName) {
    return obj[fieldName] ? null : "The '".concat(fieldName, "' field is required");
  }).filter(function (msg) {
    return msg;
  });
  var shouldWarningsBePrinted = warningMessages.length > 0;

  if (shouldWarningsBePrinted) {
    // eslint-disable-next-line no-console
    console.warn([beforeWarnings].concat((0, _toConsumableArray2.default)(warningMessages), [afterWarnings]).filter(function (line) {
      return line;
    }).join('\n'));
  }

  return !shouldWarningsBePrinted;
};

exports.checkRequiredFields = checkRequiredFields;

var getRootBasedUrl = function getRootBasedUrl(origin, rootPath, url) {
  return "".concat(origin).concat(rootPath ? "/".concat(rootPath) : '').concat(url);
};

exports.getRootBasedUrl = getRootBasedUrl;

var getKey = function getKey(viewModel, aggregateId) {
  return "".concat(viewModel, ":").concat(aggregateId);
};

exports.getKey = getKey;

var delay = function delay(timeout) {
  return new _promise.default(function (resolve) {
    return setTimeout(resolve, timeout);
  });
};

exports.delay = delay;

var makeLateResolvingPromise = function makeLateResolvingPromise() {
  if (arguments.length > 1) {
    throw new Error('Make late-resolved promise accepts only zero on one argument');
  }

  var resolvePromise = null;
  var promise = new _promise.default(function (resolve) {
    return resolvePromise = resolve;
  });
  Object.defineProperty(promise, 'resolve', {
    value: resolvePromise
  });

  if (arguments.length === 1) {
    resolvePromise(arguments.length <= 0 ? undefined : arguments[0]);
  }

  return promise;
};

exports.makeLateResolvingPromise = makeLateResolvingPromise;

var getEventTypes = function getEventTypes(viewModels, subscribers) {
  var eventTypes = {};
  (0, _keys.default)(subscribers.viewModels).forEach(function (viewModelName) {
    if (!subscribers.viewModels[viewModelName]) {
      return;
    }

    var projection = (0, _objectSpread2.default)({}, viewModels.find(function (_ref) {
      var name = _ref.name;
      return name === viewModelName;
    }).projection);
    delete projection.Init;
    (0, _keys.default)(projection).forEach(function (eventType) {
      eventTypes[eventType] = true;
    });
  });
  return (0, _keys.default)(eventTypes);
};

exports.getEventTypes = getEventTypes;

var getAggregateIds = function getAggregateIds(viewModels, subscribers) {
  if (subscribers.aggregateIds['*'] > 0) {
    return '*';
  }

  return (0, _keys.default)(subscribers.aggregateIds).filter(function (aggregateId) {
    return subscribers.aggregateIds[aggregateId];
  });
};

exports.getAggregateIds = getAggregateIds;

var createOrderedFetch = function createOrderedFetch() {
  var orderedFetchPromise = _promise.default.resolve();

  return function (url, options) {
    return new _promise.default(function (resolveResult) {
      orderedFetchPromise = orderedFetchPromise.then(
      /*#__PURE__*/
      (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!true) {
                  _context.next = 15;
                  break;
                }

                _context.prev = 1;
                _context.t0 = resolveResult;
                _context.next = 5;
                return fetch(url, options);

              case 5:
                _context.t1 = _context.sent;
                return _context.abrupt("return", (0, _context.t0)(_context.t1));

              case 9:
                _context.prev = 9;
                _context.t2 = _context["catch"](1);
                _context.next = 13;
                return new _promise.default(function (resolve) {
                  return setTimeout(resolve, 1000);
                });

              case 13:
                _context.next = 0;
                break;

              case 15:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[1, 9]]);
      })));
    });
  };
};

exports.createOrderedFetch = createOrderedFetch;